var _ = require('lodash');
var loaderUtils = require('loader-utils');

function processOptions(source, options, resourcePath) {
  if (!_.isUndefined(options.search) && !_.isUndefined(options.replace)) {
    if (!_.isUndefined(options.flags)) {
      options.search = new RegExp(options.search, options.flags);
    }

    var replace = function (w) {
      return options.replace.apply(null, [resourcePath].concat([].slice.call(arguments)));
    };

    var newSource = source.replace(options.search, replace);
    if (options.strict && (newSource === source)) {
      throw new Error('Cannot replace ' + options.search + ' → ' + replace);
    }
  } else if (options.strict) {
    throw new Error('Cannot replace: undefined search or/and option(s) → ' + JSON.stringify(options));
  }

  return newSource;
}

module.exports = function (source, map) {
  this.cacheable();

  var options = loaderUtils.getOptions(this);
  var resourcePath = this.resourcePath;

  if (_.isArray(options.multiple)) {
    options.multiple.forEach(function (suboptions) {
      suboptions.strict = (!_.isUndefined(suboptions.strict) ? suboptions.strict : options.strict);
      source = processOptions(source, suboptions, resourcePath);
    });
  } else {
    source = processOptions(source, options, resourcePath);
  }

  this.callback(null, source, map);
};
